import static org.junit.Assert.assertEquals;

import javax.xml.transform.stream.StreamSource;

import org.junit.Test;

import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.SaxonApiException;

/**
 * Reproduces unexplained slowness in Saxon grammar loading.
 */
public class TestSchemaLoading {

  private final Processor _processor;

  public TestSchemaLoading() {
    _processor = ProcessorFactory.createProcessor();
  }

  private void loadSchema(final String location) throws SaxonApiException {
    _processor.getSchemaManager().load(new StreamSource(location));
  }

  private void assertSchemaCount() {
    assertEquals(15, _processor.getUnderlyingConfiguration().getImportedNamespaces().size());
  }

  /**
   * In this case we've modified ext.xsd to import full_ifrs-cor, its fast.
   */
  @Test
  public void testFastCase() throws Exception {
    loadSchema("filing-copy/taxonomy/ext.xsd");
    assertSchemaCount();
  }


  /**
   * In this case we load full_ifrs-cor before ext.xsd, its fast too.
   */
  @Test
  public void testFastCaseB() throws Exception {
    loadSchema("http://xbrl.ifrs.org/taxonomy/2019-03-27/full_ifrs/full_ifrs-cor_2019-03-27.xsd");
    loadSchema("filing/taxonomy/ext.xsd");
    assertSchemaCount();
  }

  /**
   * In this case we load full_ifrs-cor after ext.xsd, its really slow!
   */
  @Test
  public void testSlowCase() throws Exception {
    loadSchema("filing/taxonomy/ext.xsd");
    loadSchema("http://xbrl.ifrs.org/taxonomy/2019-03-27/full_ifrs/full_ifrs-cor_2019-03-27.xsd");
    assertSchemaCount();
  }

}
